# ========================================================================
# Copyright 2018 Emory University
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========================================================================
import http.client
import json

__author__ = "Gary Lai"


class Client(object):
    def __init__(self):
        self.server_url = "https://elit.cloud/api"
        self.endpoint = '{}/{}'.format('public', 'decode')

    def decode(self, payload):
        """

        :type payload: dict
        :param payload:
        :return: result from given payload
        :rtype: json
        """

        params = json.dumps(payload)
        headers = {'Content-type': 'application/json'}
        connection = http.client.HTTPConnection(self.server_url)
        connection.request("POST", self.endpoint, params, headers)
        response = connection.getresponse()
        result = response.read()
        if response.status == 200:
            return json.loads(result)
        else:
            return {
                'status': response.status,
                'message': result
            }




# THIS FILE IS GENERATED FROM SCIPY SETUP.PY
short_version = '0.0.8'
version = '0.0.8'
full_version = '0.0.8'
git_revision = 'aab9617a521b74731ae6564d4733475828949e9f'
release = True
if not release:
    version = full_version



# ========================================================================
# Copyright 2018 Emory University
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========================================================================
"""
This repository is the elit SDK for python language. Your NLP tools implemented
the abstract class and method of this SDK can be hosted on elit.cloud.

See http://elitsdk.rtfd.io for documentation.
"""

from __future__ import absolute_import
from . import sdk, api, benchmark

__author__ = "Gary Lai"

__all__ = [
    'sdk',
    'api',
    'benchmark'
]


# ========================================================================
# Copyright 2018 Emory University
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========================================================================
# -*- coding: utf-8 -*-
"""
.. module:: elitsdk.sdk
   :synopsis: The ELIT software development kit.
.. moduleauthor:: Gary Lai
"""

import abc

__author__ = "Gary Lai"


class Component(abc.ABC):
    """
    Component is an abstract class. To deploy your method to the ELIT service,
    your class must be implemented by inheriting this class.

    Example::

        class Example(Component):
            def init(self, **kwargs):
                pass
            def decode(self, input_data, **kwargs):
                pass
            def load(self, model_path, **kwargs):
                pass
            def save(self, model_path, **kwargs):
                pass
            def train(self, trn_data, dev_data, **kwargs):
                pass


    """
    @abc.abstractmethod
    def init(self):
        """
        Implement the init method.

        """
        raise NotImplementedError("Not implemented")

    @abc.abstractmethod
    def decode(self, input_data, *args, **kwargs):
        """
        Implement the decode method.

        :param input_data: expect input for the decode function
        :param args: args for the decode method if needed
        :param kwargs: kwargs for the decode method if needed
        :return: result of your decode method
        :rtype: dict
        """
        raise NotImplementedError("Not implemented")

    @abc.abstractmethod
    def load(self, model_path, **kwargs):
        """
        Implement the method of how to load your model(s).
        If you don't need this method, leave it as pass.

        :type model_path: str
        :param model_path: root path of the model
        :param args: args for the load_model method if needed
        :param kwargs: kwargs for the load_model method if needed
        """
        raise NotImplementedError("Not implemented")

    @abc.abstractmethod
    def save(self, model_path, **kwargs):
        """
        Implement the method of how to save your model(s).
        If you don't need this method, leave it as pass.

        :type model_path: str
        :param model_path: root path of the model
        :param args: args for the save_model method if needed
        :param kwargs: kwargs for the save_model method if needed
        """
        raise NotImplementedError("Not implemented")

    @abc.abstractmethod
    def train(self, trn_data, dev_data, model_path, **kwargs):
        """
        Implement the train method.
        If you don't need this method, leave it as pass.

        :param trn_data: training data
        :param dev_data: developing data
        :param args: args for the load_model method if needed
        :param kwargs: kwargs for the load_model method if needed
        """
        raise NotImplementedError("Not implemented")



# ========================================================================
# Copyright 2018 Emory University
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========================================================================
import pytest

from elitsdk.benchmark import Timer
from elitsdk.sdk import Component
from example.example import SpaceTokenizer

__author__ = "Gary Lai"


def test_sdk():

    class TestSDK(Component):
        pass

    with pytest.raises(TypeError):
        TestSDK()


def test_abstract_class():

    class TestSDK(Component):

        def __init__(self):
            super().__init__()

        def init(self):
            super().init()

        def decode(self, input_data, **kwargs):
            super().decode(input_data, **kwargs)

        def load(self, model_path, **kwargs):
            super().load(model_path, **kwargs)

        def train(self, trn_data, dev_data, model_path, **kwargs):
            super().train(trn_data, dev_data, model_path, **kwargs)

        def save(self, model_path, **kwargs):
            super().save(model_path, **kwargs)
        
    with pytest.raises(NotImplementedError):
        test_task = TestSDK()
        test_task.decode("test")


def test_space_tokenizer():
    space_tokenizer = SpaceTokenizer()
    tokens, offset = space_tokenizer.decode("Hello, world")
    assert tokens == ['Hello,', 'world']
    assert offset == [(0, 6), (7, 12)]


def test_benchmark():
    space_tokenizer = SpaceTokenizer()
    with Timer() as time1:
        space_tokenizer.decode("Hello, world")
    with Timer() as time2:
        space_tokenizer.decode("This module implements specialized container datatypes providing alternatives to Python’s general purpose built-in containers.")

    assert isinstance(time1.runtime, float)
    assert isinstance(time2.runtime, float)
    assert time1.runtime > 0.0
    assert time2.runtime > 0.0



#!/usr/bin/env python

from __future__ import division, print_function
import os
import sys
import subprocess

from setuptools import setup, find_packages

if sys.version_info[:2] < (3, 4):
    raise RuntimeError("Python version >= 3.4 required.")

# How mature is this project? Common values are
#   3 - Alpha
#   4 - Beta
#   5 - Production/Stable
CLASSIFIERS = """\
Development Status :: 3 - Alpha
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved :: Apache Software License
Programming Language :: Python :: 3 :: Only
Programming Language :: Python :: 3.4
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Information Analysis
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

MAJOR = 0
MINOR = 0
MICRO = 8
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)

EXCLUDE_FROM_PACKAGES = ['']


# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH', 'HOME']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION


def get_version_info():
    # Adding the git rev number needs to be done inside write_version_py(),
    # otherwise the import of numpy.version messes up the build under Python 3.
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('elitsdk/version.py'):
        # must be a source distribution, use existing version file
        try:
            from elitsdk.version import git_revision as GIT_REVISION
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "elitsdk/version.py and the build directory "
                              "before building.")
    else:
        GIT_REVISION = "Unknown"

    if not ISRELEASED:
        FULLVERSION += '.dev0+' + GIT_REVISION[:7]

    return FULLVERSION, GIT_REVISION


def write_version_py(filename='elitsdk/version.py'):
    cnt = """
# THIS FILE IS GENERATED FROM SCIPY SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s
if not release:
    version = full_version
"""
    FULLVERSION, GIT_REVISION = get_version_info()

    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'full_version': FULLVERSION,
                       'git_revision': GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        a.close()


def setup_package():
    write_version_py()

    metadata = dict(
        name='elitsdk',
        url='https://github.com/elitcloud/elit-sdk-python',
        download_url='https://github.com/elitcloud/elit-sdk-python/tree/master',
        author='Gary Lai',
        author_email='gary.lai@emory.edu',
        description='The Emory Language Information Toolkit (ELIT).',
        license='ALv2',
        packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
        install_requires=[],
        tests_require=[
            'pytest',
        ],
        include_package_data=True,
        classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
    )
    metadata['version'] = get_version_info()[0]

    setup(**metadata)


if __name__ == '__main__':
    setup_package()
